/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package FunctionLayer;

import DataLayer.DataMapper;
import FunctionLayer.StanfordParser.SentimentAnalyzerTest;
import com.google.common.base.Stopwatch;
import com.google.common.collect.MapMaker;
import edu.stanford.nlp.ie.AbstractSequenceClassifier;
import edu.stanford.nlp.ie.crf.CRFClassifier;
import edu.stanford.nlp.ling.CoreLabel;
import edu.stanford.nlp.parser.lexparser.LexicalizedParser;
import edu.stanford.nlp.pipeline.Annotation;
import edu.stanford.nlp.pipeline.CoreDocument;
import edu.stanford.nlp.pipeline.StanfordCoreNLP;
import edu.stanford.nlp.tagger.maxent.MaxentTagger;
import edu.stanford.nlp.trees.GrammaticalStructureFactory;
import edu.stanford.nlp.trees.TreebankLanguagePack;
import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.Random;
import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ForkJoinPool;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author install1
 */
public class Datahandler {

    public static final long EXPIRE_TIME_IN_SECONDS = TimeUnit.SECONDS.convert(10, TimeUnit.MINUTES);
    public static final long EXPIRE_TIME_IN_SECONDS1 = TimeUnit.SECONDS.convert(10, TimeUnit.HOURS);
    public static Datahandler instance = new Datahandler();
    private static volatile Double minDistance;
    private static Annotation strAnno;
    private static Annotation strAnnoSentiment;
    private static Annotation strAnnoJMWE;
    private static CoreDocument coreDoc;
    private volatile boolean refreshMatrixFromDB;
    private static volatile int secondaryIterator = 0;
    private static volatile Double preRelationCounters = 0.0;
    private static volatile Double preRelationUserCounters = 0.0;
    private final ConcurrentMap<Integer, String> stringCache;
    private static ConcurrentMap<String, Annotation> pipelineAnnotationCache;
    private static ConcurrentMap<String, Annotation> pipelineSentimentAnnotationCache;
    private static ConcurrentMap<String, Annotation> jmweAnnotationCache;
    private static ConcurrentMap<String, CoreDocument> coreDocumentAnnotationCache;
    private static ConcurrentMap<Integer, String> conversationMatchMap;
    private LinkedHashMap<String, LinkedHashMap<String, Double>> lHMSMX = new LinkedHashMap();
    private final Stopwatch stopwatch;
    private final Stopwatch stopwatch1;
    private ForkJoinPool executor;
    private static String similar = "";
    private static String shiftReduceParserPath = "edu/stanford/nlp/models/srparser/englishSR.ser.gz";
    private static String sentimentModel = "edu/stanford/nlp/models/sentiment/sentiment.ser.gz";
    private static String lexParserEnglishRNN = "edu/stanford/nlp/models/lexparser/englishRNN.ser.gz";
    private static String taggerPath = "edu/stanford/nlp/models/pos-tagger/english-left3words/english-left3words-distsim.tagger";
    private static String nerModel = "edu/stanford/nlp/models/ner/english.all.3class.caseless.distsim.crf.ser.gz";
    private static String nerModel2 = "edu/stanford/nlp/models/ner/english.conll.4class.caseless.distsim.crf.ser.gz";
    private static String nerModel3 = "edu/stanford/nlp/models/ner/english.muc.7class.caseless.distsim.crf.ser.gz";
    private static String nerModelCaseless = "edu/stanford/nlp/models/truecase/truecasing.fast.caseless.qn.ser.gz";
    private static MaxentTagger tagger;
    private static String[] options = {"-maxLength", "100"};
    private static Properties props = new Properties();
    private static Properties propsSentiment = new Properties();
    private static GrammaticalStructureFactory gsf;
    private static LexicalizedParser lp;
    private static TreebankLanguagePack tlp;
    private static AbstractSequenceClassifier<CoreLabel> classifier;
    // set up Stanford CoreNLP pipeline
    private static StanfordCoreNLP pipeline;
    private static StanfordCoreNLP pipelineSentiment;

    public Datahandler() {
        this.stopwatch = Stopwatch.createUnstarted();
        this.stopwatch1 = Stopwatch.createStarted();
        this.stringCache = new MapMaker().concurrencyLevel(2).makeMap();
        this.jmweAnnotationCache = new MapMaker().concurrencyLevel(2).makeMap();
        this.pipelineAnnotationCache = new MapMaker().concurrencyLevel(2).makeMap();
        this.pipelineSentimentAnnotationCache = new MapMaker().concurrencyLevel(2).makeMap();
        this.coreDocumentAnnotationCache = new MapMaker().concurrencyLevel(2).makeMap();
        this.conversationMatchMap = new MapMaker().concurrencyLevel(2).makeMap();
    }

    public void shiftReduceParserInitiate() {
        //got 8 cores
        CountDownLatch cdl = new CountDownLatch(3);
        new Thread(() -> {
            try {
                classifier = CRFClassifier.getClassifierNoExceptions(nerModel);
            } catch (ClassCastException ex) {
                Logger.getLogger(Datahandler.class.getName()).log(Level.SEVERE, null, ex);
            }
            cdl.countDown();
        }).start();
        new Thread(() -> {
            propsSentiment.setProperty("parse.model", lexParserEnglishRNN);
            propsSentiment.setProperty("sentiment.model", sentimentModel);
            propsSentiment.setProperty("parse.maxlen", "90");
            propsSentiment.setProperty("threads", "25");
            propsSentiment.setProperty("pos.maxlen", "90");
            propsSentiment.setProperty("tokenize.maxlen", "90");
            propsSentiment.setProperty("ssplit.maxlen", "90");
            propsSentiment.setProperty("annotators", "tokenize,ssplit,pos,parse,sentiment"); //coref too expensive memorywise, does it need depparse?
            propsSentiment.setProperty("tokenize.options", "untokenizable=firstKeep");
            pipelineSentiment = new StanfordCoreNLP(propsSentiment);
            tagger = new MaxentTagger(taggerPath);
            cdl.countDown();
        }).start();
        new Thread(() -> {
            props.setProperty("annotators", "tokenize,ssplit,pos,lemma,parse,ner");
            props.setProperty("parse.model", shiftReduceParserPath);
            props.setProperty("parse.maxlen", "90");
            props.setProperty("parse.binaryTrees", "true");
            props.setProperty("threads", "25");
            props.setProperty("pos.maxlen", "90");
            props.setProperty("tokenize.maxlen", "90");
            props.setProperty("ssplit.maxlen", "90");
            props.setProperty("lemma.maxlen", "90");
            props.setProperty("ner.model", nerModel + "," + nerModel2 + "," + nerModel3);
            props.setProperty("ner.combinationMode", "HIGH_RECALL");
            props.setProperty("regexner.ignorecase", "true");
            props.setProperty("ner.fine.regexner.ignorecase", "true");
            props.setProperty("tokenize.options", "untokenizable=firstKeep"); //firstKeep //firstDelete
            pipeline = new StanfordCoreNLP(props);
            cdl.countDown();
        }).start();
        lp = LexicalizedParser.loadModel(lexParserEnglishRNN, options);
        tlp = lp.getOp().langpack();
        gsf = tlp.grammaticalStructureFactory();
        try {
            cdl.await();
        } catch (InterruptedException ex) {
            System.out.println("cdl await interrupted: " + ex.getLocalizedMessage() + "\n");
        }
        System.out.println("finished shiftReduceParserInitiate\n");
    }

    public static AbstractSequenceClassifier<CoreLabel> getClassifier() {
        return classifier;
    }

    public static void setClassifier(AbstractSequenceClassifier<CoreLabel> classifier) {
        Datahandler.classifier = classifier;
    }

    public void updateStringCache() {
        try {
            checkIfUpdateStrings(true);
        } catch (CustomError ex) {
            Logger.getLogger(Datahandler.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void instantiateExecutor() {
        this.executor = new ForkJoinPool(25,
                ForkJoinPool.defaultForkJoinWorkerThreadFactory,
                null, false);
    }

    public static GrammaticalStructureFactory getGsf() {
        return gsf;
    }

    public static StanfordCoreNLP getPipeline() {
        return pipeline;
    }

    public static StanfordCoreNLP getPipelineSentiment() {
        return pipelineSentiment;
    }

    public static MaxentTagger getTagger() {
        return tagger;
    }

    private Map<Integer, String> getCache() throws SQLException, IOException, CustomError {
        return DataMapper.getAllStrings();
    }

    public int getlHMSMXSize() {
        return lHMSMX.size();
    }

    public int getstringCacheSize() {
        return stringCache.size();
    }

    public void initiateMYSQL() throws SQLException, IOException {
        try {
            DataMapper.createTables();
            stringCache.putAll(getCache());
            lHMSMX = DataMapper.getAllRelationScores();
        } catch (CustomError ex) {
            Logger.getLogger(Datahandler.class
                    .getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void addHLstatsMessages() {
        ConcurrentMap<Integer, String> hlStatsMessages = new MapMaker().concurrencyLevel(2).makeMap();
        ConcurrentMap<Integer, String> strCacheLocal = stringCache;
        int hardcap = 10; //55000
        int ij = 0;
        for (String str : DataMapper.getHLstatsMessages().values()) {
            hlStatsMessages.put(ij, str);
            ij++;
            if (ij > hardcap) {
                break;
            }
        }
        hlStatsMessages.values().parallelStream().forEach(str -> {
            if (!str.startsWith("!")) {
                String orElse = strCacheLocal.values().parallelStream().filter(e -> e.equals(str)).findAny().orElse(null);
                if (orElse == null) {
                    MessageResponseHandler.getMessage(str);
                }
            }
        });
    }

    public void instantiateAnnotationMapJMWE() {
        if (!stringCache.isEmpty()) {
            ConcurrentMap<String, Annotation> jmweAnnotation = PipelineJMWESingleton.INSTANCE.getJMWEAnnotation(stringCache.values());
            for (Entry<String, Annotation> entries : jmweAnnotation.entrySet()) {
                jmweAnnotationCache.put(entries.getKey(), entries.getValue());
            }
        }
    }

    public void instantiateAnnotationMap() {
        if (!stringCache.isEmpty()) {
            ConcurrentMap<String, Annotation> Annotationspipeline = new MapMaker().concurrencyLevel(2).makeMap();
            ConcurrentMap<String, Annotation> AnnotationspipelineSentiment = new MapMaker().concurrencyLevel(2).makeMap();
            stringCache.values().parallelStream().forEach(str -> {
                Annotation strAnno = new Annotation(str);
                Annotationspipeline.put(str, strAnno);
                Annotation strAnno2 = new Annotation(str);
                AnnotationspipelineSentiment.put(str, strAnno2);
                CoreDocument CD = new CoreDocument(str);
                pipeline.annotate(CD);
                coreDocumentAnnotationCache.put(str, CD);
            });
            pipeline.annotate(Annotationspipeline.values());
            pipelineSentiment.annotate(AnnotationspipelineSentiment.values());
            Annotationspipeline.entrySet().forEach(pipelineEntry -> {
                pipelineAnnotationCache.put(pipelineEntry.getKey(), pipelineEntry.getValue());
            });
            AnnotationspipelineSentiment.entrySet().forEach(pipelineEntry -> {
                pipelineSentimentAnnotationCache.put(pipelineEntry.getKey(), pipelineEntry.getValue());
            });
        }
    }

    public synchronized void updateMatrixes() {
        refreshMatrixFromDB = false;
        if (stopwatch1.elapsed(TimeUnit.SECONDS) >= EXPIRE_TIME_IN_SECONDS1) {
            refreshMatrixFromDB = true;
            lHMSMX = DataMapper.getAllRelationScores();
            stopwatch1.reset();
        }
        if (stringCache.values().size() > 10 && !refreshMatrixFromDB) {
            ConcurrentMap<Integer, String> stringCachelocal = stringCache;
            int selectUpdate = -1;
            int iteratorCap = 25;
            LinkedHashMap<String, LinkedHashMap<String, Double>> LHMSMXLocal = lHMSMX;
            int ij2 = 0;
            for (String str : stringCachelocal.values()) {
                LinkedHashMap<String, Double> orDefault = LHMSMXLocal.getOrDefault(str, null);
                if (orDefault == null) {
                    selectUpdate = ij2;
                    break;
                }
                ij2++;
            }
            //secondaryIterator
            if (selectUpdate == -1 || selectUpdate + 1 >= stringCachelocal.size() || stringCachelocal.get(selectUpdate) == null) {
                Integer iterator = 0;
                while (iterator == 0) {
                    if (secondaryIterator >= stringCachelocal.size()) {
                        secondaryIterator = 0;
                    }
                    String get = stringCachelocal.get(secondaryIterator);
                    if (get == null) {
                        secondaryIterator++;
                    } else {
                        selectUpdate = secondaryIterator;
                        iterator++;
                    }
                }
                secondaryIterator++;
            }
            String getStringCacheStr = stringCachelocal.get(selectUpdate);
            ConcurrentMap<Integer, SimilarityMatrix> matrixUpdateMap = new MapMaker().concurrencyLevel(2).makeMap();
            ConcurrentMap<Integer, Future<SimilarityMatrix>> futures = new MapMaker().concurrencyLevel(2).makeMap();
            stringCachelocal.values().forEach((str1) -> {
                if (!getStringCacheStr.equals(str1)) {
                    boolean present = false;
                    LinkedHashMap<String, Double> orDefault = lHMSMX.getOrDefault(getStringCacheStr, null);
                    if (orDefault != null) {
                        Collection<String> strkeys = orDefault.keySet();
                        for (String strkey : strkeys) {
                            if (strkey.equals(str1)) {
                                present = true;
                                break;
                            }
                        }
                    }
                    if (!present) {
                        orDefault = lHMSMX.getOrDefault(str1, null);
                        if (orDefault != null) {
                            Collection<String> strkeys = orDefault.keySet();
                            for (String strkey : strkeys) {
                                if (strkey.equals(getStringCacheStr)) {
                                    present = true;
                                    break;
                                }
                            }
                        }
                    }
                    if (!present) {
                        LinkedHashMap<String, Double> orDefault1 = lHMSMX.getOrDefault(getStringCacheStr, null);
                        if (orDefault1 == null) {
                            orDefault1 = new LinkedHashMap<String, Double>();
                        }
                        orDefault1.put(str1, 0.0);
                        lHMSMX.put(getStringCacheStr, orDefault1);
                        SimilarityMatrix SMX = new SimilarityMatrix(getStringCacheStr, str1);
                        Callable<SimilarityMatrix> worker = new SentimentAnalyzerTest(getStringCacheStr, str1, SMX, jmweAnnotationCache.get(getStringCacheStr),
                                jmweAnnotationCache.get(str1), pipelineAnnotationCache.get(getStringCacheStr), pipelineAnnotationCache.get(str1),
                                pipelineSentimentAnnotationCache.get(getStringCacheStr), pipelineSentimentAnnotationCache.get(str1),
                                coreDocumentAnnotationCache.get(getStringCacheStr), coreDocumentAnnotationCache.get(str1));
                        futures.put(futures.size() + 1, executor.submit(worker));
                    }
                }
            });
            System.out.println("finished worker assignment, futures size: " + futures.size() + "\n");
            futures.values().parallelStream().forEach((future) -> {
                SimilarityMatrix SMX = new SimilarityMatrix("", "");
                try {
                    SMX = future.get(5, TimeUnit.SECONDS);
                } catch (InterruptedException | ExecutionException | TimeoutException ex) {
                    Logger.getLogger(Datahandler.class.getName()).log(Level.SEVERE, null, ex);
                    SMX = null;
                }
                if (SMX != null) {
                    LinkedHashMap<String, Double> getFuture = lHMSMX.getOrDefault(SMX.getPrimaryString(), null);
                    getFuture.put(SMX.getSecondaryString(), SMX.getDistance());
                    lHMSMX.put(SMX.getPrimaryString(), getFuture);
                    matrixUpdateMap.put(matrixUpdateMap.size() + 1, SMX);
                }
            });

            new Thread(() -> {
                try {
                    if (!matrixUpdateMap.isEmpty()) {
                        DataMapper.insertSementicMatrixes(matrixUpdateMap);
                        System.out.println("finished datamapper semetic insert");
                    }
                } catch (CustomError ex) {
                    Logger.getLogger(Datahandler.class
                            .getName()).log(Level.SEVERE, null, ex);
                }
            }).start();
        }
    }

    public ConcurrentMap<Integer, String> removeNonSensicalStrings(ConcurrentMap<Integer, String> strmap) {
        ConcurrentMap<Integer, String> strmapreturn = new MapMaker().concurrencyLevel(2).makeMap();
        int relationCap = 20;
        ConcurrentMap<Integer, String> strCacheLocal = stringCache.size() < 150 ? strmap : stringCache;
        ConcurrentMap<String, Annotation> localJMWEMap = getMultipleJMWEAnnotation(strmap.values());
        ConcurrentMap<String, Annotation> localPipelineAnnotation = getMultiplePipelineAnnotation(strmap.values());
        ConcurrentMap<String, Annotation> localPipelineSentimentAnnotation = getMultiplePipelineSentimentAnnotation(strmap.values());
        ConcurrentMap<String, CoreDocument> localCoreDocumentMap = getMultipleCoreDocuments(strmap.values());
        for (String str : strmap.values()) {
            ConcurrentMap<Integer, Future<SimilarityMatrix>> futures = new MapMaker().concurrencyLevel(2).makeMap();
            ConcurrentMap<Integer, HashMap<String, String>> strsmaps = new MapMaker().concurrencyLevel(2).makeMap();
            for (String str1 : strCacheLocal.values()) {
                HashMap HM1 = new HashMap();
                HM1.put(str, str1);
                if (!str.equals(str1) && !strsmaps.values().contains(HM1)) {
                    SimilarityMatrix SMX = new SimilarityMatrix(str, str1);
                    Callable<SimilarityMatrix> worker;
                    if (stringCache.size() < 150) {
                        worker = new SentimentAnalyzerTest(str, str1, SMX,
                                localJMWEMap.get(str), localJMWEMap.get(str1), localPipelineAnnotation.get(str),
                                localPipelineAnnotation.get(str1), localPipelineSentimentAnnotation.get(str),
                                localPipelineSentimentAnnotation.get(str1), localCoreDocumentMap.get(str), localCoreDocumentMap.get(str1));
                    } else {
                        worker = new SentimentAnalyzerTest(str, str1, SMX,
                                localJMWEMap.get(str), jmweAnnotationCache.get(str1), localPipelineAnnotation.get(str),
                                pipelineAnnotationCache.get(str1), localPipelineSentimentAnnotation.get(str),
                                pipelineSentimentAnnotationCache.get(str1), localCoreDocumentMap.get(str), coreDocumentAnnotationCache.get(str1));
                    }
                    HashMap HM = new HashMap();
                    HM.put(SMX.getPrimaryString(), SMX.getSecondaryString());
                    strsmaps.put(strsmaps.size() + 1, HM);
                    futures.put(futures.size() + 1, executor.submit(worker));
                }
            }
            int positiveRelationCounter = 0;
            for (Future<SimilarityMatrix> future : futures.values()) {
                try {
                    SimilarityMatrix getSMX = future.get(5, TimeUnit.SECONDS);
                    Double scoreRelationNewMsgToRecentMsg = getSMX.getDistance();
                    if (scoreRelationNewMsgToRecentMsg >= 5000.0) {
                        System.out.println("scoreRelationNewMsgToRecentMsg: " + scoreRelationNewMsgToRecentMsg + "\n");
                        positiveRelationCounter++;
                        if (positiveRelationCounter > relationCap) {
                            strmapreturn.put(strmapreturn.size() + 1, str);
                        }
                        if (positiveRelationCounter > relationCap) {
                            System.out.println("strmapreturn size: " + strmapreturn.size() + "\n");
                            break;
                        }
                    }
                } catch (InterruptedException | ExecutionException | TimeoutException ex) {
                    Logger.getLogger(Datahandler.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        }
        return strmapreturn;
    }

    public synchronized void checkIfUpdateStrings(boolean hlStatsMsg) throws CustomError {
        if (stopwatch.elapsed(TimeUnit.SECONDS) >= EXPIRE_TIME_IN_SECONDS || !stopwatch.isRunning()) {
            ConcurrentMap<Integer, String> str = MessageResponseHandler.getStr();
            str = cutContent(str, hlStatsMsg);
            str = filterContent(str);
            str = removeSlacks(str);
            System.out.println("finished removeSlacks \n" + str.size() + "\n");
            str = removeNonSensicalStrings(str);
            System.out.println("POST removeNonSensicalStrings size: " + str.size() + "\n");
            str = annotationCacheUpdate(str);
            System.out.println("annotationCacheUpdate str size POST: " + str.size() + "\n");
            ConcurrentMap<Integer, String> strf = str;
            if (!stringCache.isEmpty()) {
                new Thread(() -> {
                    try {
                        DataMapper.InsertMYSQLStrings(strf);
                    } catch (CustomError ex) {
                        Logger.getLogger(Datahandler.class
                                .getName()).log(Level.SEVERE, null, ex);
                    }
                    MessageResponseHandler.setStr(new MapMaker().concurrencyLevel(2).makeMap());
                }).start();
            } else {
                try {
                    DataMapper.InsertMYSQLStrings(strf);
                } catch (CustomError ex) {
                    Logger.getLogger(Datahandler.class
                            .getName()).log(Level.SEVERE, null, ex);
                }
                MessageResponseHandler.setStr(new MapMaker().concurrencyLevel(2).makeMap());
            }
            if (!stopwatch.isRunning()) {
                stopwatch.start();
            } else {
                stopwatch.reset();
            }
        }
    }

    public synchronized String getResponseMsg(String str, String MostRecent) throws CustomError {
        str = str.trim();
        if (str.startsWith("<@")) {
            str = str.substring(str.indexOf("> ") + 2);
        }
        SimilarityMatrix SMXreturn = new SimilarityMatrix("", "");
        ConcurrentMap<Integer, String> strCache = stringCache;
        ConcurrentMap<Integer, Future<SimilarityMatrix>> futureslocal = new MapMaker().concurrencyLevel(2).makeMap();
        ConcurrentMap<Integer, SimilarityMatrix> futurereturn = new MapMaker().concurrencyLevel(2).makeMap();
        String strF = str;
        getSingularAnnotation(strF);
        strCache.values().parallelStream().forEach((str1) -> {
            if (!strF.equals(str1)) {
                boolean present = false;
                for (String strCons : conversationMatchMap.values()) {
                    if (strCons.equals(str1)) {
                        present = true;
                        break;
                    }
                }
                if (!present) {
                    SimilarityMatrix SMX = new SimilarityMatrix(strF, str1);
                    Callable<SimilarityMatrix> worker = new SentimentAnalyzerTest(strF, str1, SMX,
                            strAnnoJMWE, jmweAnnotationCache.get(str1), strAnno,
                            pipelineAnnotationCache.get(str1), strAnnoSentiment,
                            pipelineSentimentAnnotationCache.get(str1), coreDoc, coreDocumentAnnotationCache.get(str1));
                    futureslocal.put(futureslocal.size() + 1, executor.submit(worker));
                }
            }
        });
        futureslocal.values().parallelStream().forEach((future) -> {
            SimilarityMatrix SMX = new SimilarityMatrix("", "");
            try {
                SMX = future.get(5, TimeUnit.SECONDS);
                futurereturn.put(futurereturn.size() + 1, SMX);
            } catch (InterruptedException | ExecutionException | TimeoutException ex) {
                System.out.println("ex getResponsemsg: " + ex.getMessage() + "\n");
            }
        });
        preRelationCounters = 0.0;
        preRelationUserCounters = 0.0;
        conversationMatchMap.put(conversationMatchMap.size() + 1, MostRecent);
        Double scoreRelationNewMsgToRecentMsg = 0.0;
        for (String conversationStr : conversationMatchMap.values()) {
            scoreRelationNewMsgToRecentMsg += getScoreRelationNewMsgToRecentMsg(strF, conversationStr);
        }
        boolean relatedReponse = scoreRelationNewMsgToRecentMsg >= 250;
        if (!relatedReponse) {
            conversationMatchMap = new MapMaker().concurrencyLevel(2).makeMap();
        }
        for (SimilarityMatrix SMX : futurereturn.values()) {
            Double scoreRelation = 500.0;
            Double scoreRelationLastUserMsg = SMX.getDistance();
            if (relatedReponse) {
                for (String conversationStr : conversationMatchMap.values()) {
                    scoreRelation += getScoreRelationNewMsgToRecentMsg(SMX.getSecondaryString(), conversationStr);
                }
            }
            Double totalRelation = scoreRelation + scoreRelationLastUserMsg;
            if (totalRelation > preRelationCounters + preRelationUserCounters && scoreRelationLastUserMsg > preRelationUserCounters) {
                SMXreturn = SMX;
                preRelationCounters = scoreRelation;
                preRelationUserCounters = scoreRelationLastUserMsg;
            }
        }
        System.out.println("Reached end: secondary: " + SMXreturn.getSecondaryString() + "\nPrimarY: " + SMXreturn.getPrimaryString()
                + "\nScore: " + SMXreturn.getDistance());
        return SMXreturn.getSecondaryString();
    }

    public void getSingularAnnotation(String str) {
        strAnno = new Annotation(str);
        pipeline.annotate(strAnno);
        strAnnoSentiment = new Annotation(str);
        pipelineSentiment.annotate(strAnnoSentiment);
        List<String> notactualList = new ArrayList();
        notactualList.add(str);
        ConcurrentMap<String, Annotation> jmweAnnotation = PipelineJMWESingleton.INSTANCE.getJMWEAnnotation(notactualList);
        strAnnoJMWE = jmweAnnotation.values().iterator().next();
        CoreDocument coreDocument = new CoreDocument(str);
        pipeline.annotate(coreDocument);
        coreDoc = coreDocument;
    }

    public ConcurrentMap<String, Annotation> getMultipleJMWEAnnotation(Collection<String> str) {
        ConcurrentMap<String, Annotation> jmweAnnotation = PipelineJMWESingleton.INSTANCE.getJMWEAnnotation(str);
        return jmweAnnotation;
    }

    public ConcurrentMap<String, Annotation> getMultiplePipelineAnnotation(Collection<String> str) {
        ConcurrentMap<String, Annotation> pipelineAnnotationMap = new MapMaker().concurrencyLevel(2).makeMap();
        for (String str1 : str) {
            Annotation strAnno1 = new Annotation(str1);
            pipelineAnnotationMap.put(str1, strAnno1);
        }
        pipeline.annotate(pipelineAnnotationMap.values());
        return pipelineAnnotationMap;
    }

    public ConcurrentMap<String, Annotation> getMultiplePipelineSentimentAnnotation(Collection<String> str) {
        ConcurrentMap<String, Annotation> pipelineAnnotationMap = new MapMaker().concurrencyLevel(2).makeMap();
        for (String str1 : str) {
            Annotation strAnno1 = new Annotation(str1);
            pipelineAnnotationMap.put(str1, strAnno1);
        }
        pipelineSentiment.annotate(pipelineAnnotationMap.values());
        return pipelineAnnotationMap;
    }

    public ConcurrentMap<String, CoreDocument> getMultipleCoreDocuments(Collection<String> str) {
        ConcurrentMap<String, CoreDocument> pipelineCoreDocumentAnnotations = new MapMaker().concurrencyLevel(2).makeMap();
        str.parallelStream().forEach((str1) -> {
            CoreDocument coreDocument = new CoreDocument(str1);
            pipeline.annotate(coreDocument);
            pipelineCoreDocumentAnnotations.put(str1, coreDocument);
        });
        return pipelineCoreDocumentAnnotations;
    }

    private Double getScoreRelationNewMsgToRecentMsg(String str, String mostRecentMsg) {
        SimilarityMatrix SMX = new SimilarityMatrix(str, mostRecentMsg);
        Callable<SimilarityMatrix> worker = new SentimentAnalyzerTest(str, mostRecentMsg, SMX,
                jmweAnnotationCache.get(str), jmweAnnotationCache.get(mostRecentMsg), pipelineAnnotationCache.get(str),
                pipelineAnnotationCache.get(mostRecentMsg), pipelineSentimentAnnotationCache.get(str),
                pipelineSentimentAnnotationCache.get(mostRecentMsg), coreDocumentAnnotationCache.get(str),
                coreDocumentAnnotationCache.get(mostRecentMsg));
        SimilarityMatrix callSMX = null;
        try {
            callSMX = worker.call();
        } catch (Exception ex) {
            Logger.getLogger(Datahandler.class
                    .getName()).log(Level.SEVERE, null, ex);
        }
        if (callSMX != null) {
            double smxDistance = callSMX.getDistance();
            return smxDistance;
        }
        return 0.0;
    }

    public String mostSimilar(String toBeCompared, ConcurrentMap<Integer, String> concurrentStrings, String MostRecent) {
        similar = "";
        minDistance = 6.0;
        preRelationCounters = 0.0;
        preRelationUserCounters = 0.0;
        getSingularAnnotation(toBeCompared);
        ConcurrentMap<Integer, String> similardistances = new MapMaker().concurrencyLevel(2).makeMap();
        concurrentStrings.values().parallelStream().forEach((str) -> {
            LevenshteinDistance leven = new LevenshteinDistance(toBeCompared, str);
            double distance = leven.computeLevenshteinDistance();
            if (distance <= minDistance) {
                System.out.println("distance: " + distance + "\n");
                similardistances.put(similardistances.size() + 1, str);
            }
        });
        if (similardistances.isEmpty()) {
            return null;
        }
        Double maxDistance = 0.0;
        for (String str : similardistances.values()) {
            LinkedHashMap<String, Double> orDefault = lHMSMX.getOrDefault(str, null);
            if (orDefault != null) {
                for (Entry<String, Double> defaultEntry : orDefault.entrySet()) {
                    Double value = defaultEntry.getValue();
                    String key = defaultEntry.getKey();
                    if (value > maxDistance) {
                        Double RelationScoreLastMsg = 500.0;
                        Double RelationScoreLastUserMsg = 500.0;
                        boolean foundmatch = false;
                        if (!MostRecent.isEmpty()) {
                            LinkedHashMap<String, Double> orDefault1 = lHMSMX.getOrDefault(MostRecent, null);
                            if (orDefault1 != null) {
                                Collection<String> orDefaultstrs = orDefault1.keySet();
                                for (String strs : orDefaultstrs) {
                                    if (strs.equals(key)) {
                                        RelationScoreLastMsg = orDefault1.get(key);
                                        foundmatch = true;
                                        break;
                                    }
                                }
                            }
                            if (!foundmatch) {
                                orDefault1 = lHMSMX.getOrDefault(key, null);
                                if (orDefault1 != null) {
                                    Collection<String> orDefaultstrs = orDefault1.keySet();
                                    for (String strs : orDefaultstrs) {
                                        if (strs.equals(MostRecent)) {
                                            RelationScoreLastMsg = orDefault1.get(MostRecent);
                                            foundmatch = true;
                                            break;
                                        }
                                    }
                                }
                            }
                        }
                        if (!foundmatch) {
                            RelationScoreLastMsg = getScoreRelationNewMsgToRecentMsg(key, MostRecent);
                        }
                        RelationScoreLastUserMsg = getScoreRelationNewMsgToRecentMsg(key, toBeCompared);
                        Double totalRelation = RelationScoreLastMsg + RelationScoreLastUserMsg;
                        if (totalRelation > preRelationCounters + preRelationUserCounters) {
                            if (RelationScoreLastMsg + 500 > preRelationUserCounters && RelationScoreLastUserMsg > preRelationCounters
                                    || RelationScoreLastUserMsg + 500 > preRelationCounters && RelationScoreLastMsg > preRelationUserCounters) {
                                if (RelationScoreLastMsg > preRelationCounters && RelationScoreLastUserMsg > preRelationUserCounters) {
                                    maxDistance = value;
                                    similar = defaultEntry.getKey();
                                    preRelationCounters = RelationScoreLastMsg;
                                    preRelationUserCounters = RelationScoreLastUserMsg;
                                }
                            }
                        }
                    }
                }
            }
        }
        return similar.isEmpty() ? null : similar;
    }

    public static ConcurrentMap<Integer, String> cutContent(ConcurrentMap<Integer, String> str, boolean hlStatsMsg) {
        ConcurrentMap<Integer, String> returnlist = new MapMaker().concurrencyLevel(2).makeMap();
        str.values().parallelStream().forEach(str1 -> {
            int iend = str1.indexOf("content: ");
            if (iend != -1) {
                String trs = str1.substring(iend + 9);
                returnlist.put(returnlist.size() + 1, trs.substring(0, trs.length() - 1));
            } else if (hlStatsMsg) {
                returnlist.put(returnlist.size() + 1, str1);
            }
        });
        return returnlist;
    }

    public static ConcurrentMap<Integer, String> filterContent(ConcurrentMap<Integer, String> str) {
        ConcurrentMap<Integer, String> strlistreturn = new MapMaker().concurrencyLevel(2).makeMap();
        str.values().parallelStream().forEach(str1 -> {
            if (!str1.isEmpty() && str1.length() > 3) {
                str1 = str1.trim();
                if (str1.contains("PM*")) {
                    str1 = str1.substring(str1.indexOf("PM*") + 3);
                }
                if (str1.contains("AM*")) {
                    str1 = str1.substring(str1.indexOf("AM*") + 3);
                }
                for (Character c : str1.toCharArray()) {
                    if (c == '?' || c == '°') {
                        str1 = str1.replace("?", " <:wlenny:514861023002624001> ");
                        str1 = str1.replace("°", " <:wlenny:514861023002624001> ");
                    }
                }
                if (str1.contains("(Counter-Terrorist)")) {
                    str1 = str1.replace("(Counter-Terrorist)", " ");
                }
                if (str1.contains("(Terrorist)")) {
                    str1 = str1.replace("(Terrorist)", " ");
                }
                if (str1.contains("(Spectator)")) {
                    str1 = str1.replace("(Spectator)", " ");
                }
                if (str1.contains("*DEAD*")) {
                    str1 = str1.replace("*DEAD*", " ");
                }
                if (str1.contains("{red}")) {
                    str1 = str1.replace("{red}", " ");
                }
                if (str1.contains("{orange}")) {
                    str1 = str1.replace("{orange}", " ");
                }
                if (str1.contains("{yellow}")) {
                    str1 = str1.replace("{yellow}", " ");
                }
                if (str1.contains("{green}")) {
                    str1 = str1.replace("{green}", " ");
                }
                if (str1.contains("{lightblue}")) {
                    str1 = str1.replace("{lightblue}", " ");
                }
                if (str1.contains("{blue}")) {
                    str1 = str1.replace("{blue}", " ");
                }
                if (str1.contains("{purple}")) {
                    str1 = str1.replace("{purple}", " ");
                }
                if (str1.contains("{white}")) {
                    str1 = str1.replace("{white}", " ");
                }
                if (str1.contains("{fullblue}")) {
                    str1 = str1.replace("{fullblue}", " ");
                }
                if (str1.contains("{cyan}")) {
                    str1 = str1.replace("{cyan}", " ");
                }
                if (str1.contains("{lime}")) {
                    str1 = str1.replace("{lime}", " ");
                }
                if (str1.contains("{deeppink}")) {
                    str1 = str1.replace("{deeppink}", " ");
                }
                if (str1.contains("{slategray}")) {
                    str1 = str1.replace("{slategray}", " ");
                }
                if (str1.contains("{dodgerblue}")) {
                    str1 = str1.replace("{dodgerblue}", " ");
                }
                if (str1.contains("{black}")) {
                    str1 = str1.replace("{black}", " ");
                }
                if (str1.contains("{orangered}")) {
                    str1 = str1.replace("{orangered}", " ");
                }
                if (str1.contains("{darkorchid}")) {
                    str1 = str1.replace("{darkorchid}", " ");
                }
                if (str1.contains("{pink}")) {
                    str1 = str1.replace("{pink}", " ");
                }
                if (str1.contains("{lightyellow}")) {
                    str1 = str1.replace("{lightyellow}", " ");
                }
                if (str1.contains("{chocolate}")) {
                    str1 = str1.replace("{chocolate}", " ");
                }
                if (str1.contains("{beige}")) {
                    str1 = str1.replace("{beige}", " ");
                }
                if (str1.contains("{azure}")) {
                    str1 = str1.replace("{azure}", " ");
                }
                if (str1.contains("{yellowgreen}")) {
                    str1 = str1.replace("{yellowgreen}", " ");
                }
                str1 = str1.trim();
                if (str1.length() > 2 && (!str1.startsWith("!"))) {
                    strlistreturn.put(strlistreturn.size() + 1, str1);
                }
            }
        });
        return strlistreturn;
    }

    private ConcurrentMap<Integer, String> removeSlacks(ConcurrentMap<Integer, String> str) {
        ConcurrentMap<Integer, String> strreturn = new MapMaker().concurrencyLevel(2).makeMap();
        if (stringCache.isEmpty()) {
            return str;
        }
        Collection<String> values = stringCache.values();
        str.values().parallelStream().forEach(str1 -> {
            boolean tooclosematch = false;
            for (String strVals : values) {
                LevenshteinDistance leven = new LevenshteinDistance(strVals, str1);
                double Distance = leven.computeLevenshteinDistance();
                Double maxpermittedDistance = 2.5;
                if (Distance < maxpermittedDistance) {
                    tooclosematch = true;
                    break;
                }
            }
            if (!tooclosematch) {
                strreturn.put(strreturn.size() + 1, str1);
                System.out.println("adding strreturn str1: " + str1 + "\n");
            }
        });
        return strreturn;
    }

    private ConcurrentMap<Integer, String> annotationCacheUpdate(ConcurrentMap<Integer, String> strmap) {
        ConcurrentMap<String, Annotation> jmweAnnotation = PipelineJMWESingleton.INSTANCE.getJMWEAnnotation(strmap.values());
        for (Entry<String, Annotation> jmweitr : jmweAnnotation.entrySet()) {
            jmweAnnotationCache.put(jmweitr.getKey(), jmweitr.getValue());
        }
        ConcurrentMap<String, Annotation> Annotationspipeline = new MapMaker().concurrencyLevel(2).makeMap();
        ConcurrentMap<String, Annotation> AnnotationspipelineSentiment = new MapMaker().concurrencyLevel(2).makeMap();
        strmap.values().parallelStream().forEach(str -> {
            Annotation strAnno1 = new Annotation(str);
            Annotationspipeline.put(str, strAnno1);
            Annotation strAnno2 = new Annotation(str);
            AnnotationspipelineSentiment.put(str, strAnno2);
            try {
                CoreDocument CD = new CoreDocument(str);
                pipeline.annotate(CD);
                coreDocumentAnnotationCache.put(str, CD);
            } catch (Exception e) {
                System.out.println("failed document annotation: " + e + "\n");
            }
            stringCache.put(stringCache.size() + 1, str);
        });
        System.out.println("pre iterator annotation update \n");
        pipeline.annotate(Annotationspipeline.values());
        pipelineSentiment.annotate(AnnotationspipelineSentiment.values());
        Annotationspipeline.entrySet().forEach(pipelineEntry -> {
            if (pipelineEntry != null) {
                pipelineAnnotationCache.put(pipelineEntry.getKey(), pipelineEntry.getValue());
            } else {
                System.out.println("failed pipeline cache \n");
            }
        });
        AnnotationspipelineSentiment.entrySet().forEach(pipelineEntry -> {
            if (pipelineEntry != null) {
                pipelineSentimentAnnotationCache.put(pipelineEntry.getKey(), pipelineEntry.getValue());
            } else {
                System.out.println("failed sentiment cache \n");
            }
        });
        return strmap;
    }
}
